@@ -1,6 +1,6 @@
 /* getkey.c -  Get a key from the database
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ *               2006, 2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -578,6 +578,7 @@ seckey_available( u32 *keyid )
  *   Words are delimited by white space or "()<>[]{}.@-+_,;/&!"
  *   (note that you can't search for these characters). Compare
  *   is not case sensitive.
+ * - If the userid starts with a '&' a 40 hex digits keygrip is expected.
  */
 
 int
@@ -644,7 +645,7 @@ classify_user_id( const char *name, KEYDB_SEARCH_DESC *desc )
 #endif
 
 	case '#':  /* local user id */
-            return 0; /* This is now obsolete and van't not be used anymore*/
+            return 0; /* This is now obsolete and can't not be used anymore*/
         
         case ':': /*Unified fingerprint */
             {  
@@ -669,6 +670,9 @@ classify_user_id( const char *name, KEYDB_SEARCH_DESC *desc )
             } 
             break;
            
+	case '&':  /* keygrip */
+          return 0; /* Not yet implememted. */
+
 	default:
 	    if (s[0] == '0' && s[1] == 'x') {
 		hexprefix = 1;
@@ -808,14 +812,14 @@ skip_unusable(void *dummy,u32 *keyid,PKT_user_id *uid)
  */
 
 static int
-key_byname( GETKEY_CTX *retctx, STRLIST namelist,
+key_byname( GETKEY_CTX *retctx, strlist_t namelist,
 	    PKT_public_key *pk, PKT_secret_key *sk,
 	    int secmode, int include_unusable,
             KBNODE *ret_kb, KEYDB_HANDLE *ret_kdbhd )
 {
     int rc = 0;
     int n;
-    STRLIST r;
+    strlist_t r;
     GETKEY_CTX ctx;
     KBNODE help_kb = NULL;
     
@@ -919,7 +923,7 @@ get_pubkey_byname (PKT_public_key *pk,
                    KEYDB_HANDLE *ret_kdbhd, int include_unusable )
 {
   int rc;
-  STRLIST namelist = NULL;
+  strlist_t namelist = NULL;
 
   add_to_strlist( &namelist, name );
 
@@ -935,7 +939,7 @@ get_pubkey_byname (PKT_public_key *pk,
 
       for(akl=opt.auto_key_locate;akl;akl=akl->next)
 	{
-	  unsigned char *fpr;
+	  unsigned char *fpr=NULL;
 	  size_t fpr_len;
 
 	  switch(akl->type)
@@ -1044,7 +1048,7 @@ get_pubkey_byname (PKT_public_key *pk,
 
 int
 get_pubkey_bynames( GETKEY_CTX *retctx, PKT_public_key *pk,
-		    STRLIST names, KBNODE *ret_keyblock )
+		    strlist_t names, KBNODE *ret_keyblock )
 {
     return key_byname( retctx, names, pk, NULL, 0, 1, ret_keyblock, NULL);
 }
@@ -1195,7 +1199,7 @@ get_seckey_byname2( GETKEY_CTX *retctx,
 		    PKT_secret_key *sk, const char *name, int unprotect,
 		    KBNODE *retblock )
 {
-  STRLIST namelist = NULL;
+  strlist_t namelist = NULL;
   int rc,include_unusable=1;
 
   /* If we have no name, try to use the default secret key.  If we
@@ -1228,7 +1232,7 @@ get_seckey_byname( PKT_secret_key *sk, const char *name, int unlock )
 
 int
 get_seckey_bynames( GETKEY_CTX *retctx, PKT_secret_key *sk,
-		    STRLIST names, KBNODE *ret_keyblock )
+		    strlist_t names, KBNODE *ret_keyblock )
 {
     return key_byname( retctx, names, NULL, sk, 1, 1, ret_keyblock, NULL );
 }
@@ -1486,18 +1490,23 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
 
     sig->flags.chosen_selfsig = 1; /* we chose this one */
     uid->created = 0; /* not created == invalid */
-    if ( IS_UID_REV ( sig ) ) {
+    if ( IS_UID_REV ( sig ) ) 
+      {
         uid->is_revoked = 1;
         return; /* has been revoked */
-    }
+      }
+    else
+      uid->is_revoked = 0;
 
     uid->expiredate = sig->expiredate;
 
-    if(sig->flags.expired)
+    if (sig->flags.expired)
       {
 	uid->is_expired = 1;
 	return; /* has expired */
       }
+    else
+      uid->is_expired = 0;
 
     uid->created = sig->timestamp; /* this one is okay */
     uid->selfsigversion = sig->version;
@@ -1507,12 +1516,12 @@ fixup_uidnode ( KBNODE uidnode, KBNODE signode, u32 keycreated )
     /* store the key flags in the helper variable for later processing */
     uid->help_key_usage=parse_key_usage(sig);
 
-    /* ditto or the key expiration */
-    uid->help_key_expire = 0;
+    /* ditto for the key expiration */
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE, NULL);
-    if ( p ) { 
-        uid->help_key_expire = keycreated + buffer_to_u32(p);
-    }
+    if( p && buffer_to_u32(p) )
+      uid->help_key_expire = keycreated + buffer_to_u32(p);
+    else
+      uid->help_key_expire = 0;
 
     /* Set the primary user ID flag - we will later wipe out some
      * of them to only have one in our keyblock */
@@ -1724,7 +1733,7 @@ merge_selfsigs_main(KBNODE keyblock, int *r_revoked, struct revoke_info *rinfo)
 	key_usage=parse_key_usage(sig);
 
 	p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE, NULL);
-	if ( p )
+	if( p && buffer_to_u32(p) )
 	  {
 	    key_expire = keytimestamp + buffer_to_u32(p);
 	    key_expire_seen = 1;
@@ -2128,7 +2137,7 @@ merge_selfsigs_subkey( KBNODE keyblock, KBNODE subnode )
     subpk->pubkey_usage = key_usage;
     
     p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_KEY_EXPIRE, NULL);
-    if ( p ) 
+    if ( p && buffer_to_u32(p) )
         key_expire = keytimestamp + buffer_to_u32(p);
     else
         key_expire = 0;
