@@ -80,7 +80,7 @@ static struct parse_options keyserver_opts[]=
     {NULL,0,NULL,NULL}
   };
 
-static int keyserver_work(enum ks_action action,STRLIST list,
+static int keyserver_work(enum ks_action action,strlist_t list,
 			  KEYDB_SEARCH_DESC *desc,int count,
 			  unsigned char **fpr,size_t *fpr_len,
 			  struct keyserver_spec *keyserver);
@@ -91,7 +91,7 @@ static int keyserver_work(enum ks_action action,STRLIST list,
 static size_t max_cert_size=DEFAULT_MAX_CERT_SIZE;
 
 static void
-add_canonical_option(char *option,STRLIST *list)
+add_canonical_option(char *option,strlist_t *list)
 {
   char *arg=argsplit(option);
 
@@ -941,19 +941,23 @@ direct_uri_map(const char *scheme,unsigned int is_direct)
   return 0;
 }
 
+#if GNUPG_MAJOR_VERSION == 2
+#define GPGKEYS_PREFIX "gpg2keys_"
+#else
 #define GPGKEYS_PREFIX "gpgkeys_"
+#endif
 #define GPGKEYS_CURL GPGKEYS_PREFIX "curl" EXEEXT
 #define GPGKEYS_PREFIX_LEN (strlen(GPGKEYS_CURL))
 #define KEYSERVER_ARGS_KEEP " -o \"%O\" \"%I\""
 #define KEYSERVER_ARGS_NOKEEP " -o \"%o\" \"%i\""
 
 static int 
-keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
+keyserver_spawn(enum ks_action action,strlist_t list,KEYDB_SEARCH_DESC *desc,
 		int count,int *prog,unsigned char **fpr,size_t *fpr_len,
 		struct keyserver_spec *keyserver)
 {
   int ret=0,i,gotversion=0,outofband=0;
-  STRLIST temp;
+  strlist_t temp;
   unsigned int maxlen,buflen;
   char *command,*end,*searchstr=NULL;
   byte *line=NULL;
@@ -1163,7 +1167,7 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 
     case KS_GETNAME:
       {
-	STRLIST key;
+	strlist_t key;
 
 	fprintf(spawn->tochild,"COMMAND GETNAME\n\n");
 
@@ -1185,30 +1189,31 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 
     case KS_SEND:
       {
-	STRLIST key;
+	strlist_t key;
 
 	/* Note the extra \n here to send an empty keylist block */
 	fprintf(spawn->tochild,"COMMAND SEND\n\n\n");
 
 	for(key=list;key!=NULL;key=key->next)
 	  {
-	    armor_filter_context_t afx;
-	    IOBUF buffer=iobuf_temp();
+	    armor_filter_context_t *afx;
+	    IOBUF buffer = iobuf_temp ();
 	    KBNODE block;
 
 	    temp=NULL;
 	    add_to_strlist(&temp,key->d);
 
-	    memset(&afx,0,sizeof(afx));
-	    afx.what=1;
+	    afx = new_armor_context ();
+	    afx->what = 1;
 	    /* Tell the armor filter to use Unix-style \n line
 	       endings, since we're going to fprintf this to a file
 	       that (on Win32) is open in text mode.  The win32 stdio
 	       will transform the \n to \r\n and we'll end up with the
 	       proper line endings on win32.  This is a no-op on
 	       Unix. */
-	    afx.eol[0]='\n';
-	    iobuf_push_filter(buffer,armor_filter,&afx);
+	    afx->eol[0] = '\n';
+	    push_armor_filter (afx, buffer);
+            release_armor_context (afx);
 
 	    /* TODO: Remove Comment: lines from keys exported this
 	       way? */
@@ -1318,10 +1323,14 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 			(ulong)block->pkt->pkt.public_key->keyid[0],
 			(ulong)block->pkt->pkt.public_key->keyid[1]);
 
-		fprintf(spawn->tochild,"KEY %s BEGIN\n",key->d);
+		fprintf(spawn->tochild,"KEY %08lX%08lX BEGIN\n",
+			(ulong)block->pkt->pkt.public_key->keyid[0],
+			(ulong)block->pkt->pkt.public_key->keyid[1]);
 		fwrite(iobuf_get_temp_buffer(buffer),
 		       iobuf_get_temp_length(buffer),1,spawn->tochild);
-		fprintf(spawn->tochild,"KEY %s END\n",key->d);
+		fprintf(spawn->tochild,"KEY %08lX%08lX END\n",
+			(ulong)block->pkt->pkt.public_key->keyid[0],
+			(ulong)block->pkt->pkt.public_key->keyid[1]);
 
 		iobuf_close(buffer);
 
@@ -1345,7 +1354,7 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 
     case KS_SEARCH:
       {
-	STRLIST key;
+	strlist_t key;
 
 	fprintf(spawn->tochild,"COMMAND SEARCH\n\n");
 
@@ -1402,7 +1411,7 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
       maxlen=1024;
       if(iobuf_read_line(spawn->fromchild,&line,&buflen,&maxlen)==0)
 	{
-	  ret = gpg_error_from_errno (errno);
+	  ret = gpg_error_from_syserror ();
 	  goto fail; /* i.e. EOF */
 	}
 
@@ -1494,7 +1503,7 @@ keyserver_spawn(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 }
 
 static int 
-keyserver_work(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
+keyserver_work(enum ks_action action,strlist_t list,KEYDB_SEARCH_DESC *desc,
 	       int count,unsigned char **fpr,size_t *fpr_len,
 	       struct keyserver_spec *keyserver)
 {
@@ -1564,9 +1573,9 @@ keyserver_work(enum ks_action action,STRLIST list,KEYDB_SEARCH_DESC *desc,
 }
 
 int 
-keyserver_export(STRLIST users)
+keyserver_export(strlist_t users)
 {
-  STRLIST sl=NULL;
+  strlist_t sl=NULL;
   KEYDB_SEARCH_DESC desc;
   int rc=0;
 
@@ -1596,7 +1605,7 @@ keyserver_export(STRLIST users)
 }
 
 int 
-keyserver_import(STRLIST users)
+keyserver_import(strlist_t users)
 {
   KEYDB_SEARCH_DESC *desc;
   int num=100,count=0;
@@ -1671,13 +1680,13 @@ keyserver_import_keyid(u32 *keyid,struct keyserver_spec *keyserver)
 
 /* code mostly stolen from do_export_stream */
 static int 
-keyidlist(STRLIST users,KEYDB_SEARCH_DESC **klist,int *count,int fakev3)
+keyidlist(strlist_t users,KEYDB_SEARCH_DESC **klist,int *count,int fakev3)
 {
   int rc=0,ndesc,num=100;
   KBNODE keyblock=NULL,node;
   KEYDB_HANDLE kdbhd;
   KEYDB_SEARCH_DESC *desc;
-  STRLIST sl;
+  strlist_t sl;
 
   *count=0;
 
@@ -1827,7 +1836,7 @@ keyidlist(STRLIST users,KEYDB_SEARCH_DESC **klist,int *count,int fakev3)
    usernames to refresh only part of the keyring. */
 
 int
-keyserver_refresh(STRLIST users)
+keyserver_refresh(strlist_t users)
 {
   int rc,count,numdesc,fakev3=0;
   KEYDB_SEARCH_DESC *desc;
@@ -1916,7 +1925,7 @@ keyserver_refresh(STRLIST users)
 }
 
 int
-keyserver_search(STRLIST tokens)
+keyserver_search(strlist_t tokens)
 {
   if(tokens)
     return keyserver_work(KS_SEARCH,tokens,NULL,0,NULL,NULL,opt.keyserver);
@@ -1925,10 +1934,10 @@ keyserver_search(STRLIST tokens)
 }
 
 int
-keyserver_fetch(STRLIST urilist)
+keyserver_fetch(strlist_t urilist)
 {
   KEYDB_SEARCH_DESC desc;
-  STRLIST sl;
+  strlist_t sl;
   unsigned int options=opt.keyserver_options.import_options;
 
   /* Switch on fast-import, since fetch can handle more than one
@@ -2012,7 +2021,7 @@ keyserver_import_cert(const char *name,unsigned char **fpr,size_t *fpr_len)
 	  spec=parse_keyserver_uri(url,1,NULL,0);
 	  if(spec)
 	    {
-	      STRLIST list=NULL;
+	      strlist_t list=NULL;
 
 	      add_to_strlist(&list,url);
 
@@ -2079,7 +2088,7 @@ int
 keyserver_import_name(const char *name,unsigned char **fpr,size_t *fpr_len,
 		      struct keyserver_spec *keyserver)
 {
-  STRLIST list=NULL;
+  strlist_t list=NULL;
   int rc;
 
   append_to_strlist(&list,name);
@@ -2098,7 +2107,7 @@ keyserver_import_ldap(const char *name,unsigned char **fpr,size_t *fpr_len)
 {
   char *domain;
   struct keyserver_spec *keyserver;
-  STRLIST list=NULL;
+  strlist_t list=NULL;
   int rc;
 
   append_to_strlist(&list,name);
