@@ -1,5 +1,5 @@
-/* verify.c - verify signed data
- * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+/* verify.c - Verify signed data
+ * Copyright (C) 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -24,14 +25,13 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
-#include <unistd.h> /* for isatty() */
 
+#include "gpg.h"
 #include "options.h"
 #include "packet.h"
 #include "errors.h"
 #include "iobuf.h"
 #include "keydb.h"
-#include "memory.h"
 #include "util.h"
 #include "main.h"
 #include "status.h"
@@ -54,15 +54,14 @@
 int
 verify_signatures( int nfiles, char **files )
 {
-    iobuf_t fp;
-    armor_filter_context_t afx;
-    progress_filter_context_t pfx;
+    IOBUF fp;
+    armor_filter_context_t *afx = NULL;
+    progress_filter_context_t *pfx = new_progress_context ();
     const char *sigfile;
     int i, rc;
-    STRLIST sl;
+    strlist_t sl;
 
-    memset( &afx, 0, sizeof afx);
-    /* decide whether we should handle a detached or a normal signature,
+    /* Decide whether we should handle a detached or a normal signature,
      * which is needed so that the code later can hash the correct data and
      * not have a normal signature act as detached signature and ignoring the
      * indended signed material from the 2nd file or stdin.
@@ -71,7 +70,7 @@ verify_signatures( int nfiles, char **files )
      * 3. gpg file <file2  - detached
      * 4. gpg file file2   - detached
      * The question is how decide between case 2 and 3?  The only way
-     * we can do it is by reading one byte from stdin and the unget
+     * we can do it is by reading one byte from stdin and then unget
      * it; the problem here is that we may be reading from the
      * terminal (which could be detected using isatty() but won't work
      * when under contol of a pty using program (e.g. expect)) and
@@ -86,78 +85,102 @@ verify_signatures( int nfiles, char **files )
      * that all quite easily in mainproc.c 
      */
      
-
     sigfile = nfiles? *files : NULL;
 
     /* open the signature file */
     fp = iobuf_open(sigfile);
+    if (fp && is_secured_file (iobuf_get_fd (fp)))
+      {
+        iobuf_close (fp);
+        fp = NULL;
+        errno = EPERM;
+      }
     if( !fp ) {
-        rc = gpg_error_from_errno (errno);
+        rc = gpg_error_from_syserror ();
 	log_error(_("can't open `%s': %s\n"),
                   print_fname_stdin(sigfile), strerror (errno));
-        return rc;
+        goto leave;
     }
-    handle_progress (&pfx, fp, sigfile);
+    handle_progress (pfx, fp, sigfile);
 
-    if( !opt.no_armor && use_armor_filter( fp ) )
-	iobuf_push_filter( fp, armor_filter, &afx );
+    if ( !opt.no_armor && use_armor_filter( fp ) )
+      {
+        afx = new_armor_context ();
+	push_armor_filter (afx, fp);
+      }
 
     sl = NULL;
-    for(i=1 ; i < nfiles; i++ )
+    for(i=nfiles-1 ; i > 0 ; i-- )
 	add_to_strlist( &sl, files[i] );
     rc = proc_signature_packets( NULL, fp, sl, sigfile );
     free_strlist(sl);
     iobuf_close(fp);
-    if( afx.no_openpgp_data && rc == -1 ) {
+    if( (afx && afx->no_openpgp_data && rc == -1) || rc == G10ERR_NO_DATA ) {
 	log_error(_("the signature could not be verified.\n"
 		   "Please remember that the signature file (.sig or .asc)\n"
 		   "should be the first file given on the command line.\n") );
 	rc = 0;
     }
 
+ leave:
+    release_armor_context (afx);
+    release_progress_context (pfx);
     return rc;
 }
 
 
+
 void
 print_file_status( int status, const char *name, int what )
 {
-    char *p = xmalloc (strlen(name)+10);
+    char *p = xmalloc(strlen(name)+10);
     sprintf(p, "%d %s", what, name );
     write_status_text( status, p );
-    xfree (p);
+    xfree(p);
 }
 
 
 static int
 verify_one_file( const char *name )
 {
-    iobuf_t fp;
-    armor_filter_context_t afx;
-    progress_filter_context_t pfx;
+    IOBUF fp;
+    armor_filter_context_t *afx = NULL;
+    progress_filter_context_t *pfx = new_progress_context ();
     int rc;
 
     print_file_status( STATUS_FILE_START, name, 1 );
     fp = iobuf_open(name);
+    if (fp)
+      iobuf_ioctl (fp,3,1,NULL); /* disable fd caching */
+    if (fp && is_secured_file (iobuf_get_fd (fp)))
+      {
+        iobuf_close (fp);
+        fp = NULL;
+        errno = EPERM;
+      }
     if( !fp ) {
-        rc = gpg_error_from_errno (errno);
+        rc = gpg_error_from_syserror ();
 	log_error(_("can't open `%s': %s\n"),
                   print_fname_stdin(name), strerror (errno));
 	print_file_status( STATUS_FILE_ERROR, name, 1 );
-	return rc;
+        goto leave;
     }
-    handle_progress (&pfx, fp, name);
+    handle_progress (pfx, fp, name);
 
     if( !opt.no_armor ) {
 	if( use_armor_filter( fp ) ) {
-	    memset( &afx, 0, sizeof afx);
-	    iobuf_push_filter( fp, armor_filter, &afx );
+            afx = new_armor_context ();
+	    push_armor_filter (afx, fp);
 	}
     }
 
     rc = proc_signature_packets( NULL, fp, NULL, name );
     iobuf_close(fp);
     write_status( STATUS_FILE_DONE );
+
+ leave:
+    release_armor_context (afx);
+    release_progress_context (pfx);
     return rc;
 }
 
@@ -179,7 +202,7 @@ verify_files( int nfiles, char **files )
 	    lno++;
 	    if( !*line || line[strlen(line)-1] != '\n' ) {
 		log_error(_("input line %u too long or missing LF\n"), lno );
-		return GPG_ERR_GENERAL;
+		return G10ERR_GENERAL;
 	    }
 	    /* This code does not work on MSDOS but how cares there are
 	     * also no script languages available.  We don't strip any
@@ -195,3 +218,57 @@ verify_files( int nfiles, char **files )
     }
     return 0;
 }
+
+
+
+
+/* Perform a verify operation.  To verify detached signatures, DATA_FD
+   shall be the descriptor of the signed data; for regular signatures
+   it needs to be -1.  If OUT_FP is not NULL and DATA_FD is not -1 the
+   the signed material gets written that stream. 
+
+   FIXME: OUTFP is not yet implemented.
+*/
+int
+gpg_verify (ctrl_t ctrl, int sig_fd, int data_fd, FILE *out_fp)
+{
+  int rc;
+  iobuf_t fp;
+  armor_filter_context_t *afx = NULL;
+  progress_filter_context_t *pfx = new_progress_context ();
+
+  fp = iobuf_fdopen (sig_fd, "rb");
+  if (fp && is_secured_file (sig_fd))
+    {
+      fp = NULL;
+      errno = EPERM;
+    }
+  if ( !fp )
+    {
+      rc = gpg_error_from_syserror ();
+      log_error (_("can't open fd %d: %s\n"), sig_fd, strerror (errno));
+      goto leave;
+    }
+
+  handle_progress (pfx, fp, NULL);
+
+  if ( !opt.no_armor && use_armor_filter (fp) )
+    {
+      afx = new_armor_context ();
+      push_armor_filter (afx, fp);
+    }
+
+  rc = proc_signature_packets_by_fd ( NULL, fp, data_fd );
+
+  if ( afx && afx->no_openpgp_data
+       && (rc == -1 || gpg_err_code (rc) == GPG_ERR_EOF) )
+    rc = gpg_error (GPG_ERR_NO_DATA);
+
+ leave:  
+  if (fp)
+    iobuf_close (fp);
+  release_progress_context (pfx);
+  release_armor_context (afx);
+  return rc;
+}
+
