@@ -69,7 +69,7 @@ lock_reader (int slot)
     {
       if (!pth_mutex_init (&lock_table[slot].lock))
         {
-          err = gpg_error_from_errno (errno);
+          err = gpg_error_from_syserror ();
           log_error ("error initializing mutex: %s\n", strerror (errno));
           return err;
         }
@@ -80,7 +80,7 @@ lock_reader (int slot)
   
   if (!pth_mutex_acquire (&lock_table[slot].lock, 0, NULL))
     {
-      err = gpg_error_from_errno (errno);
+      err = gpg_error_from_syserror ();
       log_error ("failed to acquire APP lock for slot %d: %s\n",
                  slot, strerror (errno));
       return err;
@@ -278,7 +278,7 @@ select_application (ctrl_t ctrl, int slot, const char *name, app_t *r_app)
   app = xtrycalloc (1, sizeof *app);
   if (!app)
     {
-      err = gpg_error_from_errno (errno);
+      err = gpg_error_from_syserror ();
       log_info ("error allocating context: %s\n", gpg_strerror (err));
       unlock_reader (slot);
       return err;
@@ -480,7 +480,7 @@ app_get_serial_and_stamp (app_t app, char **serial, time_t *stamp)
 
   buf = xtrymalloc (app->serialnolen * 2 + 1);
   if (!buf)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   for (p=buf, i=0; i < app->serialnolen; p +=2, i++)
     sprintf (p, "%02X", app->serialno[i]);
   *p = 0;
@@ -492,7 +492,7 @@ app_get_serial_and_stamp (app_t app, char **serial, time_t *stamp)
 /* Write out the application specifig status lines for the LEARN
    command. */
 gpg_error_t
-app_write_learn_status (app_t app, CTRL ctrl)
+app_write_learn_status (app_t app, ctrl_t ctrl)
 {
   gpg_error_t err;
 
@@ -574,7 +574,7 @@ app_readkey (app_t app, const char *keyid, unsigned char **pk, size_t *pklen)
 
 /* Perform a GETATTR operation.  */
 gpg_error_t 
-app_getattr (app_t app, CTRL ctrl, const char *name)
+app_getattr (app_t app, ctrl_t ctrl, const char *name)
 {
   gpg_error_t err;
 
@@ -763,7 +763,7 @@ app_writekey (app_t app, ctrl_t ctrl,
 
 /* Perform a SETATTR operation.  */
 gpg_error_t 
-app_genkey (app_t app, CTRL ctrl, const char *keynostr, unsigned int flags,
+app_genkey (app_t app, ctrl_t ctrl, const char *keynostr, unsigned int flags,
             gpg_error_t (*pincb)(void*, const char *, char **),
             void *pincb_arg)
 {
@@ -810,7 +810,7 @@ app_get_challenge (app_t app, size_t nbytes, unsigned char *buffer)
 
 /* Perform a CHANGE REFERENCE DATA or RESET RETRY COUNTER operation.  */
 gpg_error_t 
-app_change_pin (app_t app, CTRL ctrl, const char *chvnostr, int reset_mode,
+app_change_pin (app_t app, ctrl_t ctrl, const char *chvnostr, int reset_mode,
                 gpg_error_t (*pincb)(void*, const char *, char **),
                 void *pincb_arg)
 {
