@@ -32,7 +32,7 @@
 
 #include "gpgsm.h"
 
-#define set_error(e,t) assuan_set_error (ctx, ASSUAN_ ## e, (t))
+#define set_error(e,t) assuan_set_error (ctx, gpg_error (e), (t))
 
 
 /* The filepointer for status message used in non-server mode */
@@ -44,6 +44,7 @@ struct server_local_s {
   int message_fd;
   int list_internal;
   int list_external;
+  int list_to_output;           /* Write keylistings to the output fd. */
   certlist_t recplist;
   certlist_t signerlist;
   certlist_t default_recplist; /* As set by main() - don't release. */
@@ -73,6 +74,22 @@ strcpy_escaped_plus (char *d, const char *s)
 }
 
 
+/* Skip over options.  
+   Blanks after the options are also removed. */
+static char *
+skip_options (const char *line)
+{
+  while (spacep (line))
+    line++;
+  while ( *line == '-' && line[1] == '-' )
+    {
+      while (*line && !spacep (line))
+        line++;
+      while (spacep (line))
+        line++;
+    }
+  return (char*)line;
+}
 
 
 /* Check whether the option NAME appears in LINE */
@@ -83,12 +100,14 @@ has_option (const char *line, const char *name)
   int n = strlen (name);
 
   s = strstr (line, name);
+  if (s && s >= skip_options (line))
+    return 0;
   return (s && (s == line || spacep (s-1)) && (!s[n] || spacep (s+n)));
 }
 
 
 static void 
-close_message_fd (CTRL ctrl)
+close_message_fd (ctrl_t ctrl)
 {
   if (ctrl->server_local->message_fd != -1)
     {
@@ -99,24 +118,24 @@ close_message_fd (CTRL ctrl)
 
 
 static int
-option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
+option_handler (assuan_context_t ctx, const char *key, const char *value)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
 
   if (!strcmp (key, "include-certs"))
     {
       int i = *value? atoi (value) : -1;
       if (ctrl->include_certs < -2)
-        return ASSUAN_Parameter_Error;
+        return gpg_error (GPG_ERR_ASS_PARAMETER);
       ctrl->include_certs = i;
     }
-  else   if (!strcmp (key, "display"))
+  else if (!strcmp (key, "display"))
     {
       if (opt.display)
         free (opt.display);
       opt.display = strdup (value);
       if (!opt.display)
-        return ASSUAN_Out_Of_Core;
+        return out_of_core ();
     }
   else if (!strcmp (key, "ttyname"))
     {
@@ -124,7 +143,7 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
         free (opt.ttyname);
       opt.ttyname = strdup (value);
       if (!opt.ttyname)
-        return ASSUAN_Out_Of_Core;
+        return out_of_core ();
     }
   else if (!strcmp (key, "ttytype"))
     {
@@ -132,7 +151,7 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
         free (opt.ttytype);
       opt.ttytype = strdup (value);
       if (!opt.ttytype)
-        return ASSUAN_Out_Of_Core;
+        return out_of_core ();
     }
   else if (!strcmp (key, "lc-ctype"))
     {
@@ -140,7 +159,7 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
         free (opt.lc_ctype);
       opt.lc_ctype = strdup (value);
       if (!opt.lc_ctype)
-        return ASSUAN_Out_Of_Core;
+        return out_of_core ();
     }
   else if (!strcmp (key, "lc-messages"))
     {
@@ -148,7 +167,7 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
         free (opt.lc_messages);
       opt.lc_messages = strdup (value);
       if (!opt.lc_messages)
-        return ASSUAN_Out_Of_Core;
+        return out_of_core ();
     }
   else if (!strcmp (key, "list-mode"))
     {
@@ -169,15 +188,24 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
           ctrl->server_local->list_external = 1;
         }
       else
-        return ASSUAN_Parameter_Error;
+        return gpg_error (GPG_ERR_ASS_PARAMETER);
+    }
+  else if (!strcmp (key, "list-to-output"))
+    {
+      int i = *value? atoi (value) : 0;
+      ctrl->server_local->list_to_output = i;
     }
   else if (!strcmp (key, "with-validation"))
     {
       int i = *value? atoi (value) : 0;
       ctrl->with_validation = i;
     }
+  else if (!strcmp (key, "with-key-data"))
+    {
+      opt.with_key_data = 1;
+    }
   else
-    return ASSUAN_Invalid_Option;
+    return gpg_error (GPG_ERR_UNKNOWN_OPTION);
 
   return 0;
 }
@@ -186,9 +214,9 @@ option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
 
 
 static void
-reset_notify (ASSUAN_CONTEXT ctx)
+reset_notify (assuan_context_t ctx)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
 
   gpgsm_release_certlist (ctrl->server_local->recplist);
   gpgsm_release_certlist (ctrl->server_local->signerlist);
@@ -201,9 +229,9 @@ reset_notify (ASSUAN_CONTEXT ctx)
 
 
 static void
-input_notify (ASSUAN_CONTEXT ctx, const char *line)
+input_notify (assuan_context_t ctx, const char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
 
   ctrl->autodetect_encoding = 0;
   ctrl->is_pem = 0;
@@ -219,9 +247,9 @@ input_notify (ASSUAN_CONTEXT ctx, const char *line)
 }
 
 static void
-output_notify (ASSUAN_CONTEXT ctx, const char *line)
+output_notify (assuan_context_t ctx, const char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
 
   ctrl->create_pem = 0;
   ctrl->create_base64 = 0;
@@ -245,9 +273,9 @@ output_notify (ASSUAN_CONTEXT ctx, const char *line)
   client has to take care of this.  All RECIPIENT commands are
   cumulative until a RESET or an successful ENCRYPT command.  */
 static int 
-cmd_recipient (ASSUAN_CONTEXT ctx, char *line)
+cmd_recipient (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int rc;
 
   rc = gpgsm_add_to_certlist (ctrl, line, 0, &ctrl->server_local->recplist, 0);
@@ -268,7 +296,7 @@ cmd_recipient (ASSUAN_CONTEXT ctx, char *line)
                    line, NULL);
     }
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 /*  SIGNER <userID>
@@ -288,9 +316,9 @@ cmd_recipient (ASSUAN_CONTEXT ctx, char *line)
   Note that this command returns an INV_RECP status which is a bit
   strange, but they are very similar.  */
 static int 
-cmd_signer (ASSUAN_CONTEXT ctx, char *line)
+cmd_signer (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int rc;
 
   rc = gpgsm_add_to_certlist (ctrl, line, 1,
@@ -312,7 +340,7 @@ cmd_signer (ASSUAN_CONTEXT ctx, char *line)
                    "0",
                   line, NULL);
     }
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -330,9 +358,9 @@ cmd_signer (ASSUAN_CONTEXT ctx, char *line)
   have been done while setting the recipients.  The input and output
   pipes are closed. */
 static int 
-cmd_encrypt (ASSUAN_CONTEXT ctx, char *line)
+cmd_encrypt (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   certlist_t cl;
   int inp_fd, out_fd;
   FILE *out_fp;
@@ -340,14 +368,14 @@ cmd_encrypt (ASSUAN_CONTEXT ctx, char *line)
 
   inp_fd = assuan_get_input_fd (ctx);
   if (inp_fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
   out_fd = assuan_get_output_fd (ctx);
   if (out_fd == -1)
-    return set_error (No_Output, NULL);
+    return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
 
   out_fp = fdopen ( dup(out_fd), "w");
   if (!out_fp)
-    return set_error (General_Error, "fdopen() failed");
+    return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
   
   /* Now add all encrypt-to marked recipients from the default
      list. */
@@ -371,7 +399,7 @@ cmd_encrypt (ASSUAN_CONTEXT ctx, char *line)
   close_message_fd (ctrl);
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 /* DECRYPT
@@ -382,23 +410,23 @@ cmd_encrypt (ASSUAN_CONTEXT ctx, char *line)
   no need to ask the client for a protecting passphrase - GpgAgent
   does take care of this by requesting this from the user. */
 static int 
-cmd_decrypt (ASSUAN_CONTEXT ctx, char *line)
+cmd_decrypt (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int inp_fd, out_fd;
   FILE *out_fp;
   int rc;
 
   inp_fd = assuan_get_input_fd (ctx);
   if (inp_fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
   out_fd = assuan_get_output_fd (ctx);
   if (out_fd == -1)
-    return set_error (No_Output, NULL);
+    return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
 
   out_fp = fdopen ( dup(out_fd), "w");
   if (!out_fp)
-    return set_error (General_Error, "fdopen() failed");
+    return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
   rc = gpgsm_decrypt (ctrl, inp_fd, out_fp); 
   fclose (out_fp);
 
@@ -407,7 +435,7 @@ cmd_decrypt (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -421,22 +449,22 @@ cmd_decrypt (ASSUAN_CONTEXT ctx, char *line)
   the signed material and the client must provide it.
   */
 static int 
-cmd_verify (ASSUAN_CONTEXT ctx, char *line)
+cmd_verify (assuan_context_t ctx, char *line)
 {
   int rc;
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int fd = assuan_get_input_fd (ctx);
   int out_fd = assuan_get_output_fd (ctx);
   FILE *out_fp = NULL;
 
   if (fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
 
   if (out_fd != -1)
     {
       out_fp = fdopen ( dup(out_fd), "w");
       if (!out_fp)
-        return set_error (General_Error, "fdopen() failed");
+        return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
     }
 
   rc = gpgsm_verify (assuan_get_pointer (ctx), fd,
@@ -449,7 +477,7 @@ cmd_verify (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -459,9 +487,9 @@ cmd_verify (ASSUAN_CONTEXT ctx, char *line)
   set by OUTPUT.  With "--detached" specified, a detached signature is
   created (surprise).  */
 static int 
-cmd_sign (ASSUAN_CONTEXT ctx, char *line)
+cmd_sign (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int inp_fd, out_fd;
   FILE *out_fp;
   int detached;
@@ -469,16 +497,16 @@ cmd_sign (ASSUAN_CONTEXT ctx, char *line)
 
   inp_fd = assuan_get_input_fd (ctx);
   if (inp_fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
   out_fd = assuan_get_output_fd (ctx);
   if (out_fd == -1)
-    return set_error (No_Output, NULL);
+    return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
 
   detached = has_option (line, "--detached"); 
 
   out_fp = fdopen ( dup(out_fd), "w");
   if (!out_fp)
-    return set_error (General_Error, "fdopen() failed");
+    return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
 
   rc = gpgsm_sign (assuan_get_pointer (ctx), ctrl->server_local->signerlist,
                    inp_fd, detached, out_fp);
@@ -489,7 +517,7 @@ cmd_sign (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -500,14 +528,14 @@ cmd_sign (ASSUAN_CONTEXT ctx, char *line)
   the certificate but not of the entire chain.  It is possible to
   import expired certificates.  */
 static int 
-cmd_import (ASSUAN_CONTEXT ctx, char *line)
+cmd_import (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int rc;
   int fd = assuan_get_input_fd (ctx);
 
   if (fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
 
   rc = gpgsm_import (assuan_get_pointer (ctx), fd);
 
@@ -516,23 +544,36 @@ cmd_import (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
+/* EXPORT [--data [--armor|--base64]] [--] pattern
+
+ */
+
 static int 
-cmd_export (ASSUAN_CONTEXT ctx, char *line)
+cmd_export (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int fd = assuan_get_output_fd (ctx);
   FILE *out_fp;
   char *p;
-  STRLIST list, sl;
-
-  if (fd == -1)
-    return set_error (No_Output, NULL);
+  strlist_t list, sl;
+  int use_data;
   
-  /* break the line down into an STRLIST */
+  use_data = has_option (line, "--data");
+
+  if (use_data)
+    {
+      /* We need to override any possible setting done by an OUTPUT command. */
+      ctrl->create_pem = has_option (line, "--armor");
+      ctrl->create_base64 = has_option (line, "--base64");
+    }
+
+  line = skip_options (line);
+
+  /* Break the line down into an strlist_t. */
   list = NULL;
   for (p=line; *p; line = p)
     {
@@ -546,7 +587,7 @@ cmd_export (ASSUAN_CONTEXT ctx, char *line)
           if (!sl)
             {
               free_strlist (list);
-              return ASSUAN_Out_Of_Core;
+              return out_of_core ();
             }
           sl->flags = 0;
           strcpy_escaped_plus (sl->d, line);
@@ -555,17 +596,36 @@ cmd_export (ASSUAN_CONTEXT ctx, char *line)
         }
     }
 
-  out_fp = fdopen ( dup(fd), "w");
-  if (!out_fp)
+  if (use_data)
     {
-      free_strlist (list);
-      return set_error (General_Error, "fdopen() failed");
+      out_fp = assuan_get_data_fp (ctx);
+      if (!out_fp)
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_ASS_GENERAL, "no data stream");
+        }
+      gpgsm_export (ctrl, list, out_fp);
+    }
+  else
+    {
+      if (fd == -1)
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
+        }
+      out_fp = fdopen ( dup(fd), "w");
+      if (!out_fp)
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
+        }
+      
+      gpgsm_export (ctrl, list, out_fp);
+      fclose (out_fp);
     }
 
-  gpgsm_export (ctrl, list, out_fp);
-  fclose (out_fp);
   free_strlist (list);
-  /* close and reset the fd */
+  /* Close and reset the fds. */
   close_message_fd (ctrl);
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
@@ -574,14 +634,14 @@ cmd_export (ASSUAN_CONTEXT ctx, char *line)
 
 
 static int 
-cmd_delkeys (ASSUAN_CONTEXT ctx, char *line)
+cmd_delkeys (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   char *p;
-  STRLIST list, sl;
+  strlist_t list, sl;
   int rc;
 
-  /* break the line down into an STRLIST */
+  /* break the line down into an strlist_t */
   list = NULL;
   for (p=line; *p; line = p)
     {
@@ -595,7 +655,7 @@ cmd_delkeys (ASSUAN_CONTEXT ctx, char *line)
           if (!sl)
             {
               free_strlist (list);
-              return ASSUAN_Out_Of_Core;
+              return out_of_core ();
             }
           sl->flags = 0;
           strcpy_escaped_plus (sl->d, line);
@@ -612,7 +672,7 @@ cmd_delkeys (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -622,42 +682,37 @@ cmd_delkeys (ASSUAN_CONTEXT ctx, char *line)
    Set the file descriptor to read a message which is used with
    detached signatures */
 static int 
-cmd_message (ASSUAN_CONTEXT ctx, char *line)
+cmd_message (assuan_context_t ctx, char *line)
 {
-  char *endp;
+  int rc;
   int fd;
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
 
-  if (strncmp (line, "FD=", 3))
-    return set_error (Syntax_Error, "FD=<n> expected");
-  line += 3;
-  if (!digitp (line))
-    return set_error (Syntax_Error, "number required");
-  fd = strtoul (line, &endp, 10);
-  if (*endp)
-    return set_error (Syntax_Error, "garbage found");
+  rc = assuan_command_parse_fd (ctx, line, &fd);
+  if (rc)
+    return rc;
   if (fd == -1)
-    return set_error (No_Input, NULL);
-
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
   ctrl->server_local->message_fd = fd;
   return 0;
 }
 
-
+/* LISTKEYS [<patterns>]
+   DUMPKEYS [<patterns>]
+   LISTSECRETKEYS [<patterns>]
+   DUMPSECRETKEYS [<patterns>]
+*/
 static int 
-do_listkeys (ASSUAN_CONTEXT ctx, char *line, int mode)
+do_listkeys (assuan_context_t ctx, char *line, int mode)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
-  FILE *fp = assuan_get_data_fp (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
+  FILE *fp;
   char *p;
-  STRLIST list, sl;
+  strlist_t list, sl;
   unsigned int listmode;
   gpg_error_t err;
 
-  if (!fp)
-    return set_error (General_Error, "no data stream");
-  
-  /* break the line down into an STRLIST */
+  /* Break the line down into an strlist. */
   list = NULL;
   for (p=line; *p; line = p)
     {
@@ -671,7 +726,7 @@ do_listkeys (ASSUAN_CONTEXT ctx, char *line, int mode)
           if (!sl)
             {
               free_strlist (list);
-              return ASSUAN_Out_Of_Core;
+              return out_of_core ();
             }
           sl->flags = 0;
           strcpy_escaped_plus (sl->d, line);
@@ -680,6 +735,21 @@ do_listkeys (ASSUAN_CONTEXT ctx, char *line, int mode)
         }
     }
 
+  if (ctrl->server_local->list_to_output)
+    {
+      if ( assuan_get_output_fd (ctx) == -1 )
+        return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
+      fp = fdopen (assuan_get_output_fd (ctx), "w");
+      if (!fp)
+        return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
+    }
+  else
+    {
+      fp = assuan_get_data_fp (ctx);
+      if (!fp)
+        return set_error (GPG_ERR_ASS_GENERAL, "no data stream");
+    }
+  
   ctrl->with_colons = 1;
   listmode = mode; 
   if (ctrl->server_local->list_internal)
@@ -688,21 +758,38 @@ do_listkeys (ASSUAN_CONTEXT ctx, char *line, int mode)
     listmode |= (1<<7);
   err = gpgsm_list_keys (assuan_get_pointer (ctx), list, fp, listmode);
   free_strlist (list);
-  return map_to_assuan_status (err);
+  if (ctrl->server_local->list_to_output)
+    {
+      fclose (fp);
+      assuan_close_output_fd (ctx);
+    }
+  return err;
 }
 
 static int 
-cmd_listkeys (ASSUAN_CONTEXT ctx, char *line)
+cmd_listkeys (assuan_context_t ctx, char *line)
 {
   return do_listkeys (ctx, line, 3);
 }
 
 static int 
-cmd_listsecretkeys (ASSUAN_CONTEXT ctx, char *line)
+cmd_dumpkeys (assuan_context_t ctx, char *line)
+{
+  return do_listkeys (ctx, line, 259);
+}
+
+static int 
+cmd_listsecretkeys (assuan_context_t ctx, char *line)
 {
   return do_listkeys (ctx, line, 2);
 }
 
+static int 
+cmd_dumpsecretkeys (assuan_context_t ctx, char *line)
+{
+  return do_listkeys (ctx, line, 258);
+}
+
 
 /* GENKEY
 
@@ -710,23 +797,23 @@ cmd_listsecretkeys (ASSUAN_CONTEXT ctx, char *line)
    certificate request to the output.
  */
 static int 
-cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
+cmd_genkey (assuan_context_t ctx, char *line)
 {
-  CTRL ctrl = assuan_get_pointer (ctx);
+  ctrl_t ctrl = assuan_get_pointer (ctx);
   int inp_fd, out_fd;
   FILE *out_fp;
   int rc;
 
   inp_fd = assuan_get_input_fd (ctx);
   if (inp_fd == -1)
-    return set_error (No_Input, NULL);
+    return set_error (GPG_ERR_ASS_NO_INPUT, NULL);
   out_fd = assuan_get_output_fd (ctx);
   if (out_fd == -1)
-    return set_error (No_Output, NULL);
+    return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
 
   out_fp = fdopen ( dup(out_fd), "w");
   if (!out_fp)
-    return set_error (General_Error, "fdopen() failed");
+    return set_error (GPG_ERR_ASS_GENERAL, "fdopen() failed");
   rc = gpgsm_genkey (ctrl, inp_fd, out_fp);
   fclose (out_fp);
 
@@ -734,7 +821,7 @@ cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
 
-  return map_to_assuan_status (rc);
+  return rc;
 }
 
 
@@ -743,11 +830,11 @@ cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
 
 /* Tell the assuan library about our commands */
 static int
-register_commands (ASSUAN_CONTEXT ctx)
+register_commands (assuan_context_t ctx)
 {
   static struct {
     const char *name;
-    int (*handler)(ASSUAN_CONTEXT, char *line);
+    int (*handler)(assuan_context_t, char *line);
   } table[] = {
     { "RECIPIENT",     cmd_recipient },
     { "SIGNER",        cmd_signer },
@@ -761,7 +848,9 @@ register_commands (ASSUAN_CONTEXT ctx)
     { "OUTPUT",        NULL }, 
     { "MESSAGE",       cmd_message },
     { "LISTKEYS",      cmd_listkeys },
+    { "DUMPKEYS",      cmd_dumpkeys },
     { "LISTSECRETKEYS",cmd_listsecretkeys },
+    { "DUMPSECRETKEYS",cmd_dumpsecretkeys },
     { "GENKEY",        cmd_genkey },
     { "DELKEYS",       cmd_delkeys },
     { NULL }
@@ -785,7 +874,7 @@ gpgsm_server (certlist_t default_recplist)
 {
   int rc;
   int filedes[2];
-  ASSUAN_CONTEXT ctx;
+  assuan_context_t ctx;
   struct server_control_s ctrl;
   static const char hello[] = ("GNU Privacy Guard's S/M server "
                                VERSION " ready");
@@ -793,23 +882,23 @@ gpgsm_server (certlist_t default_recplist)
   memset (&ctrl, 0, sizeof ctrl);
   gpgsm_init_default_ctrl (&ctrl);
 
-  /* For now we use a simple pipe based server so that we can work
-     from scripts.  We will later add options to run as a daemon and
-     wait for requests on a Unix domain socket */
+  /* We use a pipe based server so that we can work from scripts.
+     assuan_init_pipe_server will automagically detect when we are
+     called with a socketpair and ignore FIELDES in this case. */
   filedes[0] = 0;
   filedes[1] = 1;
   rc = assuan_init_pipe_server (&ctx, filedes);
   if (rc)
     {
       log_error ("failed to initialize the server: %s\n",
-                 assuan_strerror(rc));
+                 gpg_strerror (rc));
       gpgsm_exit (2);
     }
   rc = register_commands (ctx);
   if (rc)
     {
       log_error ("failed to the register commands with Assuan: %s\n",
-                 assuan_strerror(rc));
+                 gpg_strerror(rc));
       gpgsm_exit (2);
     }
   if (opt.verbose || opt.debug)
@@ -862,14 +951,14 @@ gpgsm_server (certlist_t default_recplist)
         }
       else if (rc)
         {
-          log_info ("Assuan accept problem: %s\n", assuan_strerror (rc));
+          log_info ("Assuan accept problem: %s\n", gpg_strerror (rc));
           break;
         }
       
       rc = assuan_process (ctx);
       if (rc)
         {
-          log_info ("Assuan processing failed: %s\n", assuan_strerror (rc));
+          log_info ("Assuan processing failed: %s\n", gpg_strerror (rc));
           continue;
         }
     }
@@ -878,6 +967,7 @@ gpgsm_server (certlist_t default_recplist)
   ctrl.server_local->recplist = NULL;
   gpgsm_release_certlist (ctrl.server_local->signerlist);
   ctrl.server_local->signerlist = NULL;
+  xfree (ctrl.server_local);
 
   assuan_deinit_server (ctx);
 }
@@ -966,7 +1056,7 @@ get_status_string ( int no )
 
 
 gpg_error_t
-gpgsm_status2 (CTRL ctrl, int no, ...)
+gpgsm_status2 (ctrl_t ctrl, int no, ...)
 {
   gpg_error_t err = 0;
   va_list arg_ptr;
@@ -1015,7 +1105,7 @@ gpgsm_status2 (CTRL ctrl, int no, ...)
     }
   else 
     {
-      ASSUAN_CONTEXT ctx = ctrl->server_local->assuan_ctx;
+      assuan_context_t ctx = ctrl->server_local->assuan_ctx;
       char buf[950], *p;
       size_t n;
 
@@ -1032,8 +1122,7 @@ gpgsm_status2 (CTRL ctrl, int no, ...)
             *p++ = *text++;
         }
       *p = 0;
-      err = map_assuan_err (assuan_write_status (ctx,
-                                                 get_status_string (no), buf));
+      err = assuan_write_status (ctx, get_status_string (no), buf);
     }
 
   va_end (arg_ptr);
@@ -1041,13 +1130,13 @@ gpgsm_status2 (CTRL ctrl, int no, ...)
 }
 
 gpg_error_t
-gpgsm_status (CTRL ctrl, int no, const char *text)
+gpgsm_status (ctrl_t ctrl, int no, const char *text)
 {
   return gpgsm_status2 (ctrl, no, text, NULL);
 }
 
 gpg_error_t
-gpgsm_status_with_err_code (CTRL ctrl, int no, const char *text,
+gpgsm_status_with_err_code (ctrl_t ctrl, int no, const char *text,
                             gpg_err_code_t ec)
 {
   char buf[30];
@@ -1059,67 +1148,3 @@ gpgsm_status_with_err_code (CTRL ctrl, int no, const char *text,
     return gpgsm_status2 (ctrl, no, buf, NULL);
 }
 
-#if 0
-/*
- * Write a status line with a buffer using %XX escapes.  If WRAP is >
- * 0 wrap the line after this length.  If STRING is not NULL it will
- * be prepended to the buffer, no escaping is done for string.
- * A wrap of -1 forces spaces not to be encoded as %20.
- */
-void
-write_status_text_and_buffer ( int no, const char *string,
-                               const char *buffer, size_t len, int wrap )
-{
-    const char *s, *text;
-    int esc, first;
-    int lower_limit = ' ';
-    size_t n, count, dowrap;
-
-    if( !statusfp )
-	return;  /* not enabled */
-    
-    if (wrap == -1) {
-        lower_limit--;
-        wrap = 0;
-    }
-
-    text = get_status_string (no);
-    count = dowrap = first = 1;
-    do {
-        if (dowrap) {
-            fprintf (statusfp, "[GNUPG:] %s ", text );
-            count = dowrap = 0;
-            if (first && string) {
-                fputs (string, statusfp);
-                count += strlen (string);
-            }
-            first = 0;
-        }
-        for (esc=0, s=buffer, n=len; n && !esc; s++, n-- ) {
-            if ( *s == '%' || *(const byte*)s <= lower_limit 
-                           || *(const byte*)s == 127 ) 
-                esc = 1;
-            if ( wrap && ++count > wrap ) {
-                dowrap=1;
-                break;
-            }
-        }
-        if (esc) {
-            s--; n++;
-        }
-        if (s != buffer) 
-            fwrite (buffer, s-buffer, 1, statusfp );
-        if ( esc ) {
-            fprintf (statusfp, "%%%02X", *(const unsigned char*)s );
-            s++; n--;
-        }
-        buffer = s;
-        len = n;
-        if ( dowrap && len )
-            putc ( '\n', statusfp );
-    } while ( len );
-
-    putc ('\n',statusfp);
-    fflush (statusfp);
-}
-#endif
